/**
 * Copyright (c) 2011 Metacert. All rights reserved.
 */

Cu.import("resource://metacert/common/common.js");
Cu.import("resource://metacert/common/uninstallService.js");
Cu.import("resource://metacert/progressListener.js");
Cu.import("resource://metacert/label/labelService.js");

/**
 * Controls the browser overlay.
 */
MetacertChrome.Overlay = {
  PREF_METACERT_ENABLED : Metacert.PrefBranch + "blocking.enabled",
  PREF_METACERT_TOOLBARBTN_ADDED : Metacert.PrefBranch + "toolbaritem.added",

  /* Logger for this object. */
  _logger : null,
  /* latest managed event listener id */
  _latestManagedEventListenerId : null,
  /* managed event listeners */
  _managedEventListeners : null,

  /**
   * Initializes the object.
   */
  init : function() {
    this._logger = Metacert.getLogger("MetacertChrome.Overlay");
    this._logger.trace("init");

    this._addToolbarButton();
  },

  /**
   * Add the toolbar button on the first run.
   */
  _addToolbarButton : function() {
    let buttonInstalled =
      Metacert.Application.prefs.get(this.PREF_METACERT_TOOLBARBTN_ADDED).value;

    if (!buttonInstalled) {
      let toolbar = document.getElementById("nav-bar");
      let newCurrentSet = null;

      if (-1 != toolbar.currentSet.indexOf("urlbar-container")) {
         newCurrentSet = toolbar.currentSet.replace(
           /urlbar-container/, "metacert-toolbar-item,urlbar-container");
      } else {
         newCurrentSet = toolbar.currentSet + ",metacert-toolbar-item";
      }

      toolbar.setAttribute("currentset", newCurrentSet);
      toolbar.currentSet = newCurrentSet;
      document.persist("nav-bar", "currentset");

      try {
        BrowserToolboxCustomizeDone(true);
      } catch (e) {
        MetacertChrome.Overlay._logger.error(e);
      }

      Metacert.Application.prefs.setValue(this.PREF_METACERT_TOOLBARBTN_ADDED,
        true);
    }
  },

  /**
   * Uninitializes the object.
   */
  uninit : function() {
    this._logger.trace("uninit");
  },

  _openSettings : function() {
    this._logger.trace("_openSettings");
    let settingsDialog =
      window.openDialog("chrome://metacert/content/settings.xul",
        "metacert-settings", "chrome,centerscreen,resizable=no");

    settingsDialog.focus();
  },

  _openLink : function(aEvent, aLink) {
    let url = "";

    switch (aLink) {
      case "classify":
        let encodedUrl = encodeURIComponent(window.content.document.URL);
        url = "http://metacert.com/submit?uri=" + encodedUrl;
      break;
      case "feedback":
        url = "https://metacert.com/feedback";
      break;
      case "about":
        url = "http://metacert.com/metasurf";
      break;
    }

    if (url.length > 0) {
      window.openUILinkIn(url, 'tab');
      aEvent.stopPropagation();
    }
  }
};

window.addEventListener(
  "load", function() { MetacertChrome.Overlay.init(); }, false);
window.addEventListener(
  "unload", function() { MetacertChrome.Overlay.uninit(); }, false);
