/**
 * Copyright (c) 2011 Metacert. All rights reserved.
 */

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://metacert/common/common.js");
Cu.import("resource://metacert/progressListener.js");

MetacertChrome.Settings = {
  PREF_METACERT_ENABLED : Metacert.PrefBranch + "blocking.enabled",
  /* Logger for this object. */
  _logger : null,
  _blockedUI : null,
  _stringBundle : null,
  _hostname : 'chrome://metacert',
  _formSubmitURL :  null,
  _httprealm : 'User Auth',
  _username : "metacert",
  _btnFocused : false,

  /**
   * Initializes the object.
   */
  init : function() {
    this._logger = Metacert.getLogger("MetacertChrome.Settings");
    this._logger.trace("init");

    this._stringBundle =
      Cc["@mozilla.org/intl/stringbundle;1"].
        getService(Ci.nsIStringBundleService).
          createBundle("chrome://metacert/locale/locale.properties");
    if (Metacert.Application.prefs.get(this.PREF_METACERT_ENABLED).value ==
      true) {
      document.getElementById("metacert-settings-description").textContent =
        this._stringBundle.
          GetStringFromName("metacert.settings.description.enabled.label");
      document.getElementById("metacert-settings-description2").textContent =
        this._stringBundle.
          GetStringFromName("metacert.settings.description.enabled.label2");
      document.getElementById("metacert-settings-state-label").value =
        this._stringBundle.
          GetStringFromName("metacert.settings.state.enabled.label");
      document.getElementById("metacert-settings-state-button").label =
        this._stringBundle.
          GetStringFromName("metacert.settings.disable.button");
    }
  },

  _switchState : function(aEvent) {
    this._logger.trace("_switchState");

    if (Metacert.Application.prefs.get(this.PREF_METACERT_ENABLED).value ==
      false) {
      this._showEnableSettingsUI();
    } else {
      this._showDisableSettingsUI();
    }
  },

  _showEnableSettingsUI : function () {
    this._blockedUI = true;
    this._btnFocused = false;
    document.getElementById("metacert-settings-initial").collapsed = true;
    document.getElementById("metacert-settings-disable").collapsed = true;
    document.getElementById("metacert-settings-enable").collapsed = false;
    document.getElementById("metacert-settings-password").focus();
  },

  _showDisableSettingsUI : function () {
    this._blockedUI = false;
    this._btnFocused = false;
    document.getElementById("metacert-settings-initial").collapsed = true;
    document.getElementById("metacert-settings-enable").collapsed = true;
    document.getElementById("metacert-settings-disable").collapsed = false;
    document.getElementById("metacert-settings-disable-password").focus();
  },

  _enableBlocking : function () {
    let pwd = document.getElementById("metacert-settings-password").value;
    let pwdConfirmation = document.
      getElementById("metacert-settings-confirm-password").value;

    if(pwd == pwdConfirmation && pwd != "") {
      //it is only one user called metacert - clean it if it was already there
      this._deleteAllUsers();
      this._addUsertoLoginManager(this._username, pwd);
      Metacert.Application.prefs.setValue(this.PREF_METACERT_ENABLED, true);
      window.close();
    } else {
      document.getElementById("metacert-settings-password").value = "";
      document.getElementById("metacert-settings-confirm-password").value = "";
      document.getElementById("metacert-settings-spacer").hidden = true;
      document.getElementById("metacert-settings-password-warning").hidden = false;
    }
  },

  _disableBlocking : function () {
    let pwd = document.getElementById("metacert-settings-disable-password").value;
    if (this._isUserValid(pwd)) {
      Metacert.Application.prefs.setValue(this.PREF_METACERT_ENABLED, false);
      window.close();
    } else {
      document.getElementById("metacert-settings-disable-password").value = "";
      document.getElementById("metacert-settings-incorrect-password").hidden = false;
    }
  },

  _addUsertoLoginManager : function (aUsername, aPassword) {
    let nsLoginInfo =
      new Components.Constructor("@mozilla.org/login-manager/loginInfo;1",
        Ci.nsILoginInfo, "init");
    let userLoginInfo = new nsLoginInfo(this._hostname, null, this._httprealm,
      aUsername, aPassword, "", "");
    let passwordManager = Cc["@mozilla.org/login-manager;1"].
      getService(Components.interfaces.nsILoginManager);
    passwordManager.addLogin(userLoginInfo);
  },

  _deleteAllUsers : function () {
    try {
      // Get Login Manager
      let loginManager = Cc["@mozilla.org/login-manager;1"].
        getService(Components.interfaces.nsILoginManager);

      // Find users for the given parameters
      let logins = loginManager.findLogins({}, this._hostname,
        this._formSubmitURL, this._httprealm);

      // Delete each user from returned array of nsILoginInfo objects
      for (let i = 0; i < logins.length; i++) {
        loginManager.removeLogin(logins[i]);
      }
    }
    catch(ex) {
    //This will only happen if there is no nsILoginManager component class
    }
  },

  _onKeyPressed : function(aEvent) {
    switch (aEvent.keyCode) {
      case KeyEvent.DOM_VK_ENTER:
      case KeyEvent.DOM_VK_RETURN:
        if (this._btnFocused == false) {
          if(this._blockedUI == true) {
            MetacertChrome.Settings._enableBlocking();
          } else if(this._blockedUI == false) {
            MetacertChrome.Settings._disableBlocking();
          } else {
            MetacertChrome.Settings._switchState();
          }
          aEvent.stopPropagation();
          aEvent.preventDefault();
        }
        break;
    }
  },

  _setFocus : function (aValue) {
    this._btnFocused = aValue;
  },

  /**
   * Validate User
   */
  _isUserValid : function(aPassword) {
    let password;
    try {
      // Get Login Manager
      let loginManager = Cc["@mozilla.org/login-manager;1"].
        getService(Components.interfaces.nsILoginManager);

      // Find users for the given parameters
      let logins = loginManager.findLogins({}, this._hostname,
        this._formSubmitURL, this._httprealm);

      // Find user from returned array of nsILoginInfo objects
      for (let i = 0; i < logins.length; i++) {
        if (logins[i].username == this._username) {
          password = logins[i].password;
          break;
        }
      }
      //User is valid
      if (password == aPassword) {
        return true;
      }
    }
    catch(ex) {
    //This will only happen if there is no nsILoginManager component class
    }
    return false;
  }
};

window.addEventListener(
  "load", function() { MetacertChrome.Settings.init(); }, false);
