/**
 * Copyright (c) 2011 Metacert. All rights reserved.
 */

var EXPORTED_SYMBOLS = [];

const Cu = Components.utils;

Cu.import("resource://metacert/common/common.js");
Cu.import("resource://metacert/helpers/objectHelper.js");

/**
 * Represents a label object.
 */
Metacert.LabelDTO = function(aLabelHost, aLabelLastUpdate, aLabelCertificate, aLabelLastRequest) {
  this._init(aLabelHost, aLabelLastUpdate, aLabelCertificate, aLabelLastRequest);
}

/* DTO Schema. */
Metacert.LabelDTO.SCHEMA = {
  NAME: "Label",
  PROPERTIES: {
    "host":        { type: "string",  xml: "" },
    "lastUpdate":  { type: "string",  xml: "" },
    "certificate": { type: "string",  xml: "" },
    "lastRequest": { type: "string",  xml: "" }
  }
};

Metacert.LabelDTO.prototype = {
  /* Logger for this object. */
  _logger : null,

  /**
   * Used to determine the type of the object
   */
  name : Metacert.LabelDTO.SCHEMA.NAME,

  /**
   * Initializes the object.
   * @param aLabelHost the label host.
   * @param aLabelLastUpdate last time updated from server.
   * @param aLabelCertificate the label certificate.
   * @param aLabelLastRequest last time requested by user.
   */
  _init : function(aLabelHost, aLabelLastUpdate, aLabelCertificate, aLabelLastRequest) {
    this._logger = Metacert.getLogger("Metacert.LabelDTO");
    this._logger.trace("_init");

    this.set("host", aLabelHost);
    this.set("lastUpdate", aLabelLastUpdate);
    this.set("certificate", aLabelCertificate);
    this.set("lastRequest", aLabelLastRequest);
  },

  /**
   * Returns the value of the given property.
   * @param aProperty the property name.
   * @return the property value.
   */
  get : function(aProperty) {
    this._logger.debug("get");

    return this["_" + aProperty];
  },

  /**
   * Sets the given property to the given value, checking that the value is of
   * the correct type for the property
   * @param aProperty the property name.
   * @param aValue the property value.
   */
  set : function(aProperty, aValue) {
    this._logger.debug("set");

    var type = Metacert.LabelDTO.SCHEMA.PROPERTIES[aProperty].type;

    aValue = Metacert.ObjectHelper.getTypedValue(type, aValue);
    this["_" + aProperty] = aValue;
  }
};
