/**
 * Copyright (c) 2011 Metacert. All rights reserved.
 */

var EXPORTED_SYMBOLS = [];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://metacert/common/common.js");
Cu.import("resource://metacert/label/labelDAO.js");
Cu.import("resource://metacert/apis/metacertApi.js");
Cu.import("resource://gre/modules/Services.jsm");

/**
 * The Metacert.LabelService
 */
Metacert.LabelService = {
  PREF_METACERT_GETUPDATES_DATEFROM : Metacert.PrefBranch + "getUpdates.dateFrom",

  /* Logger for this object. */
  _logger : null,
  _ioService: null,
  _labelsList: null,
  _labelsWhiteList: null,

  /**
   * Initialize the resource.
   */
  _init : function() {
    this._logger = Metacert.getLogger("Metacert.LabelService");
    this._logger.debug("init");

    this._ioService = Services.io;
    this._console = Services.console;
    this._labelsList = new Array();
    this._labelsWhiteList = new Array();
    //load the blacklist in memory
    this.getLabels();
    let that = this;
    let dateFrom = Metacert.Application.prefs.
      get(that.PREF_METACERT_GETUPDATES_DATEFROM).value;
    if (dateFrom == "") {
      dateFrom = Metacert.getUnixTime(new Date());
      Metacert.Application.prefs.setValue(that.PREF_METACERT_GETUPDATES_DATEFROM,
        dateFrom.toString());
    }
    MetacertApi.getUpdates(Metacert.LabelService._getUpdatesCallback, dateFrom);

    const GET_UPDATES_RATE = 1000 * 60 * 60 * 24; // 1 DAY

    this._getUpdatesTimer = Cc["@mozilla.org/timer;1"].
      createInstance(Ci.nsITimer);
    this._getUpdatesTimer.initWithCallback({
      notify : function () {
        let dateFrom = Metacert.Application.prefs.
          get(that.PREF_METACERT_GETUPDATES_DATEFROM).value;
        MetacertApi.getUpdates(Metacert.LabelService._getUpdatesCallback,
          dateFrom);
      }
    }, GET_UPDATES_RATE, Ci.nsITimer.TYPE_REPEATING_SLACK);
  },

  isOnBlackList : function(aDomainList, aCallback, aElement) {
    let onBlackList = false;
    let onWhiteList = false;
    let domainIndex, previousChar, partialHost;

    // First check if the URL should be filtered at all. This prevents the
    // filter operation to happen on the blank page about:blank and chrome
    // URLs. Then see if it's any of the pages that need an automatic redirect
    // on our side.
    try{
      if (aDomainList && aDomainList.length > 0) {
        let uri = this._ioService.newURI(aDomainList, null, null);
        let host = uri.host.toLowerCase().replace("www.","");

        if(this._labelsList) {
          let labelsListLength = this._labelsList.length;

          for (let i = 0; i < labelsListLength; i++) {
            domainIndex = host.indexOf(this._labelsList[i]);
            //if previous char is a point, if the whole string matches/or it doesn't match at all set a point
            previousChar = (domainIndex > 0) ? host[domainIndex-1] : ".";
            partialHost = host.substring(domainIndex, host.length);
            if (-1 != domainIndex && previousChar == "."
                && partialHost == this._labelsList[i]) {
              onBlackList = true;
              break;
            }
          }
        }

        if (!onBlackList) {
          //Check white list
          if(this._labelsWhiteList) {
            let labelsWhiteListLength = this._labelsWhiteList.length;

            for (let i = 0; i < labelsWhiteListLength; i++) {
              domainIndex = host.indexOf(this._labelsWhiteList[i]);
              //if previous char is a point, if the whole string matches/or it doesn't match at all set a point
              previousChar = (domainIndex > 0) ? host[domainIndex-1] : ".";
              partialHost = host.substring(domainIndex, host.length);
              if (-1 != domainIndex && previousChar == "."
                  && partialHost == this._labelsWhiteList[i]) {
                onWhiteList = true;
                break;
              }
            }
          }
          if (!onWhiteList) {
            MetacertApi.getInfo(Metacert.LabelService._getInfoCallback, host, aCallback, aElement);
          }
        }

        if (onBlackList || onWhiteList){
          aCallback(onBlackList, aElement);
        }
      }
    }
    catch (e) {
    }
  },

  /**
   * Get Info callback method, list of labelDTOs retrieval
   * @param aResult the API response
   */
  _getInfoCallback : function(aLabelListDTO, aCallback, aElement) {
    let onBlackList = false;
    if (null != aLabelListDTO) {
      let certificate, host;
      for (let j = 0; j < aLabelListDTO.length; j++) {
        certificate = aLabelListDTO[j].get("certificate");
        host = aLabelListDTO[j].get("host");
        // if present, add the host to memory
        if (certificate == "xxx") {
          Metacert.LabelService._labelsList.push(host);
          onBlackList = true;
        } else {
          if (certificate == "not-xxx") {
            Metacert.LabelService._labelsWhiteList.push(host);
          }
        }
        Metacert.LabelService.insertLabel(aLabelListDTO[j]);
      }
    }
    aCallback(onBlackList, aElement);
  },

  /**
   * Get Updates callback method, list of labelDTOs to update
   * @param aResult the API response
   */
  _getUpdatesCallback : function(aResult) {
    if (null != aResult) {
      if (null != aResult.dateFrom) {
        Metacert.Application.prefs.setValue(
          Metacert.LabelService.PREF_METACERT_GETUPDATES_DATEFROM,
            (aResult.dateFrom).toString());
      }
      if (null != aResult.labelDTOsList) {
        let listLength = aResult.labelDTOsList.length;
        let  host, index, certificate;
        for (let j = 0; j < listLength; j++) {
          certificate = aResult.labelDTOsList[j].get("certificate");
          host = aResult.labelDTOsList[j].get("host");
          // if present, update the host on database
          if (certificate == "not-xxx") {
            index = Metacert.LabelService._labelsList.indexOf(host);
            // Remove label only if it must not longer be blocked and it's on current blacklist
            if (index > -1) {
              Metacert.LabelService._labelsList.splice(index, 1);
              Metacert.LabelService._labelsWhiteList.push(host);
            }
          } else {
            if (certificate == "xxx") {
              index = Metacert.LabelService._labelsWhiteList.indexOf(host);
              // Remove label only if it must not longer be on unblocked and it's on current whitelist
              if (index > -1) {
                Metacert.LabelService._labelsWhiteList.splice(index, 1);
                Metacert.LabelService._labelsList.push(host);
              }
            }
          }
          Metacert.LabelService.strictUpdateLabel(aResult.labelDTOsList[j]);
        }
      }
    }
  },

  /**
   * Inserts a label in the DB.
   * @param aLabelDTO.
   * @param aCallback the callback.
   */
  insertLabel : function(aLabelDTO) {
    this._logger.debug("insertLabel");

    Metacert.LabelDAO.insertLabel(aLabelDTO, null);
  },

  updateLabel : function(aLabelDTO) {
    this._logger.debug("updateLabel");

    Metacert.LabelDAO.updateLabel(aLabelDTO, null);
  },

  strictUpdateLabel : function(aLabelDTO) {
    this._logger.debug("strictUpdateLabel");

    Metacert.LabelDAO.strictUpdateLabel(aLabelDTO, null);
  },

  /**
   * Gets the labels list.
   */
  getLabels : function () {
    this._logger.trace("getLabels");

    var that = this;
    var callback = function(aLabels) {
      that._getLabelsCallback(aLabels);
    };

    Metacert.LabelDAO.selectLabels(callback);
  },

  /**
   * Get the labels list callback.
   * @param aLabels all the labels.
   * @param aCallback the callback method.
   */
  _getLabelsCallback : function(aLabels) {
    Metacert.LabelService._logger.trace("_getLabelsCallback");

    let labelsList = new Array();
    let labelsWhiteList = new Array();
    let certificate, host;

    for (let i = 0; i < aLabels.length; i++) {
      certificate = aLabels[i].get("certificate");
      host = aLabels[i].get("host");
      if (certificate == "xxx") {
        labelsList.push(host);
      } else {
        if (certificate == "not-xxx") {
          labelsWhiteList.push(host);
        }
      }
    }

    Metacert.LabelService._labelsList = labelsList;
    Metacert.LabelService._labelsWhiteList = labelsWhiteList;
  }

};

/**
 * Constructor.
 */
(function() { this._init(); }).apply(Metacert.LabelService);
